/*************************************************************************************************************
1st November 2006


This program (mlogit_endogd.prg) estimates a multinomial logit model with 3 alternatives with a dummy endogenous 
regressor. A Probit model is specified for this dummy endogenous regressor

It allows for endogeneity through a common factor unobservable variable that is integrated out in the 
likelihood function.

The integration is carried out using random draws (rndn) with anthithetics. 

This program generates its own data so that one can try the program !!!!!!

The results of a small montecarlo analysis are in mlogit_endogd_montecarlo.prg and mlogit_endogd_montecarlo.out
See mlogit_endogd_montecarlo2.prg and mlogit_endogd_montecarlo2.out for a program that uses a more sophisticated
random number generator rndCLn instead of rndn.

However I have found that results using Halton sequences are better than those using random draws. See:
mlogit_endogd_h.prg; mlogit_endogd_montecarlo3.prg; mlogit_endogd_montecarlo3.out 

**************************************************************************************************************/





new;

library optmum;
#include hessian.src;
#include grad2.g;

output file = estima12.out reset;

/* Enter the number of simulation */
/* Introducir numero par */
r=100;


@----------------------- Files names ------------------------------------@
/* USER, enter here the names of the files to save the 
the coefficients and the variance covariance matrix */


ficoef = "coef12.fmt";
fimvc = "mvc12.fmt";


@------------------------- Loading data ---------------------------------@
/* 

Angel, aqui va la parte de leer los datos de la ESCA y la prima... 
Pero en este fichero voy a utilizar datos completamente simulados... 
asi que lo que hago en esta parte es fabricar estos datos simulados

d=1 significa seguro privado
y1=1 va al gp publico, 0 caso contrario
y2=1 va al esp publico, 0 caso contrario
y3=1 va a otra cosa, 0 caso contrario
p=prima
theta la vble no observable que figura en las dos ecuaciones
x otro regresor
*/

@!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!@
@Generation the data to try the model !!!!!!!!!!!!!!!@
@!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!@

n=3000;
theta=rndn(n,1);
p=rndn(n,1);
x=rndn(n,1);

@Genero los datos inventados del probit del seguro@
probd1=cdfn(x-p+theta);
d=rndu(n,1).<probd1;
@Fijate, d ha de ser 0 o 1@

@Genero los datos inventados de la utilizacion@
proby2=exp(x-d+theta)./(1+exp(x-d+theta)+exp(-x+d-theta));
proby3=exp(-x+d-theta)./(1+exp(x-d+theta)+exp(-x+d-theta));
proby1=1-proby2-proby3;
uni_aleat=rndu(n,1);
y=1+(uni_aleat.>=proby1)+(uni_aleat.>=(proby1+proby2));
y1=y.==1;
y2=y.==2;
y3=y.==3;

@This is to check that the probabilities of the generated data make sense@

"Estimated probabilities using the generated data (health care use)";
sumc(y1)/n;
sumc(y2)/n;
sumc(y3)/n;

"Real probabilities (health care use)";
sumc(proby1)/n;
sumc(proby2)/n;
sumc(proby3)/n;

/*
 We should have variables y1, y2, y3, d(insurance status), and p(premium) and the rest of covariates (x) 
namesxp= regressors for the probit. It must include the "premium" and constant term
namesy2= regressors for the specialist. It must include the insurance status "d" and constant term
namesy3= regressors fo the no public use. It must includes the insurance status "d" and constant term
*/


namesxp={ constan, x, p }; 
namesz2={ constan, x, d };
namesz3={ constan, x, d };

@initial conditions for the betas@


condxp={ 0, 
       1, 
       -1 };

condz2={ 0, 
       1, 
       -1 };


condz3={ 0, 
       -1, 
       1 };

@names for the weight of the unobservable variable in the covariates of y2 and y3@ 
names_wei={ loady2, loady3 };

@Enter their initial conditions as specified in Namesvcm. Do not introduce variaces@
cond_v={ 1, -1 };



@---------- End of Loading data------------------------------@


@----------------------- Loading Variables -------------------------------@

/*
open f1 = ^dataset for read;
call makevars(readr(f1,rowsf(f1)),0,namevar);
close(f1);


y1=mergevar(use1); 
y2=mergevar(use2);
y3=mergevar(use3);
*/


constan=ones(rows(y1),1);

xp=mergevar(namesxp);
z2=mergevar(namesz2);
z3=mergevar(namesz3);

@ Number of observations and parameters for each choice@
nobs=rows(y1);
kxp=cols(xp);
kz2=cols(z2);
kz3=cols(z3);



@---------------------------------------------------------------------@
@Getting the random draws for the simulation@
@This is the unobservable variable that must be integrated out@
@We will use antitheics, so we obtain half of the draws and the compute the rest using minus@

rndseed 1;

@In the line below, we create a matrix of independent normal(0,1) with dimension Nx(half number of simulations)@
tempu=rndn(nobs,r/2);
@This is the antithetics@
bigu=tempu~(-tempu);




@-------------------Partition the data according to different events----------@


@Selecting matrices for event y1==1 and d==1@

xp_y1d1=selif(xp,y1 .and d);
z2_y1d1=selif(z2,y1 .and d);
z3_y1d1=selif(z3,y1 .and d);
u_y1d1=selif(bigu,y1 .and d);


@Selecting matrices for event y1==1 and d==0@

xp_y1d0=selif(xp,y1 .and d.==0);
z2_y1d0=selif(z2,y1 .and d.==0);
z3_y1d0=selif(z3,y1 .and d.==0);
u_y1d0=selif(bigu,y1 .and d.==0);


@Selecting matrices for event y2==1 and d==1@

xp_y2d1=selif(xp,y2 .and d);
z2_y2d1=selif(z2,y2 .and d);
z3_y2d1=selif(z3,y2 .and d);
u_y2d1=selif(bigu,y2 .and d);


@Selecting matrices for event y2==1 and d==0@

xp_y2d0=selif(xp,y2 .and d.==0);
z2_y2d0=selif(z2,y2 .and d.==0);
z3_y2d0=selif(z3,y2 .and d.==0);
u_y2d0=selif(bigu,y2 .and d.==0);

@Selecting matrices for event y3==1 and d==1@

xp_y3d1=selif(xp,y3 .and d);
z2_y3d1=selif(z2,y3 .and d);
z3_y3d1=selif(z3,y3 .and d);
u_y3d1=selif(bigu,y3 .and d);


@Selecting matrices for event y3==1 and d==0@

xp_y3d0=selif(xp,y3 .and d.==0);
z2_y3d0=selif(z2,y3 .and d.==0);
z3_y3d0=selif(z3,y3 .and d.==0);
u_y3d0=selif(bigu,y3 .and d.==0);

@---------------------------------------------------------------------@
@Computing how many observations we have in each event@
obs_y1d1=rows(xp_y1d1);
obs_y1d0=rows(xp_y1d0);
obs_y2d1=rows(xp_y2d1);
obs_y2d0=rows(xp_y2d0);
obs_y3d1=rows(xp_y3d1);
obs_y3d0=rows(xp_y3d0);

if nobs ne obs_y1d1+obs_y1d0+obs_y2d1+obs_y2d0+obs_y3d1+obs_y3d0;
Print "There is something going wrong with the number of observations";
end;
endif;





@---------------------- Empieza el calculo de estadisticos ---------------@
"";
"We are trying to estimate a multinomial logit with endogenous probit";
"To account for endogeneity, we integrate out an unobservable variable that affects both the probit and the multinomial";
"The number of simulations is ";;r;
"";
"---------------------------Descriptive statistics-------------------------";
/* Esto lo podemos hacer cuando tengamos un dataset de gauss con las vbles
call dstat(dataset,0);
*/
@--------------------------Initial conditions--------------------------------@

@We stack in the same vector the initial conditions for the maximization of the likelihood@
theta0=condxp|condz2|condz3|cond_v;

@Here we enter the names of the parameters that are maximized @

 _opparnm=namesxp
    |namesz2
    |namesz3
    |names_wei;

@---------------------- Globals for the Optimization ------------------------@
@These are things of the optimization routine, they would be different in stata@

@for optmum@
_opgtol=1e-5;
_opalgr=2;
__title="Multinomial with endogeneity";
__output=1;
add=1;


@----------------------------- Minimizacion -------------------------------@

"-------------------------------------------------------------------------";
"We are about to start the minimization";
"-------------------------------------------------------------------------";
_fmtnv={"*.*lg" 15 8};
__fmtcv={"-*.*s" 12 8};

print"============================================================";
print"        Initial Conditions for the Minimization             ";
print"============================================================";
print"Parameter       Value     ";
print"------------------------------------------------------------";
call printfmt(_opparnm~theta0,0~1);
print"------------------------------------------------------------";
print;
"";
"";


"The value of the objective function at the initial condition is ";
mle(theta0);


@here we start the optimization of the likelihood function which is called mle@
@the initial conditions are called theta0@

{thetaest,nf,g,retcode}=optprt(optmum(&mle,theta0));
save ^ficoef =thetaest;

@Esto es un chequeo de Gauss para saber que la optimizacion esta bien@
if retcode ne 0;
print " WARNING, RETCODE IS DIFFERENT FROM ZERO ";
end;
endif;



@-----We compute the VCV matrix of the parameter estimates----------@

add=0;
jac=grad2(&mle,thetaest,mle(thetaest),0); /* Jacobian */
add=1;
inf=jac'*jac;  /*Information matrix */
invinf=invpd(inf);
vcm=invinf;            /* outer product of gradients */
std=sqrt(diag(vcm));
"";
save ^fimvc =vcm;



@----------------------- Showing the results -----------------------@

thetashow=thetaest;
noms=namesxp
    |namesz2
    |namesz3
    |names_wei;
stdshow=std;

__fmtnv={"*.*lg" 15 8};
__fmtcv={"-*.*s" 12 8};

"Standard error computed using Outer product of the gradient";
print"============================================================";
print"                    Estimation Results                     ";
print"============================================================";
print"Parameter       Estimation     Std. Error         Z-Stat";
print"------------------------------------------------------------";

call printfmt(noms~thetashow~stdshow~(thetashow./stdshow),0~1~1~1);
print"------------------------------------------------------------";
print "--------------------------------------------------------";


end;

@*************************************************************************@
/*************************    PROCEDURES   *******************************/
@*************************************************************************@

/*************************************************************************
                          log likelihood
**************************************************************************/


proc mle(theta);

@Primero se definen las variables locales a la funcion de verosimilitud@
local loglike, betaxp, betaz2, betaz3, beta_lo2, beta_lo3;
local aaa, bbb, ccc, pd1, pd0, ddd2, ddd3, fff2, fff3, ggg2, ggg3;
local py2, py2d1_n, py2d1, ly2d1;
local      py2d0_n, py2d0, ly2d0;
local py3, py3d0_n, py3d0, ly3d0;
local      py3d1_n, py3d1, ly3d1;
local py1, py1d1_n, py1d1, ly1d1;
local      py1d0_n, py1d0, ly1d0; 

@Getting the parameters@
/*
The row vector beta includes all the parameters of the model. Here I split
them in the betas of the different parts of the models, as well as the load
factors of the unobservable (beta_lo2 and beta_lo3) */

betaxp=theta[1:kxp,1];
betaz2=theta[kxp+1:kxp+kz2,1];
betaz3=theta[kxp+kz2+1:kxp+kz2+kz3,1];
beta_lo2=theta[kxp+kz2+kz3+1,1];
beta_lo3=theta[kxp+kz2+kz3+2,1];

@--------------------------------------------------@
@Getting the log likelihood of event y2==1 and d==1@
@--------------------------------------------------@

/*
aaa= the index of the probit, that is, the standard x beta. It is a row vector with dimension the 
     number of people in this event
bbb= we create a matrix of ones with dimension the (number of people in this event)x(number of simulations)
     This matrix of ones is multiplied row by row by the probit index (aaa). So in each row we will have 
     the index of the probit xbeta of the individual of that row repeated across all the columns
 
ccc= we sum the index of the probit with the unobservable variable (u_y2d1)

pd1= we apply the function FI (cumulative distribution function of the normal) to obtain the probability of
     d=1 as a function of both the xbeta and the unobservable variable!!!!! Notice that the dimension of this matrix is 
(number of people in this event)x(number of simulations), because we have a different probability for each individual
and simulation

ddd2 and ddd3 = Notice that to compute p(y2=1) of the multinomial logit we need both beta2*Z and beta3*Z 
               (beta1 is 0 because we take it as the default category). This is computed in ddd2 and ddd3

fff2 and fff3= we use the same trick as before with the matrix of ones. So, in each row we have the index 
of the multinomial logit repeated in all the columns

ggg2 and ggg3 = we add the unobservable variable to the index of the multinomial logit. Notice that 
                we have to multiply by the corresponding weighting factor 


py2 is the probabability of y2==1 with the multinomial logit. Notice that both the index and the unobservable
enter through ggg2 and ggg3. Notice that the dimension of this matrix is 
(number of people in this event)x(number of simulations), because we have a different probability for each individual
and simulation

py2d1_n= we multimply each element (i,j) of the matrix py2 by the corresponding element (i,j) of the matrix py2 
So The resulting matrix is dimension (number of people in this event)x(number of simulations). 

py2d1 = This is a row vector, with the number of rows equals the number of people in this event.
 Each row is the mean of the row py2_d1. Here we are integrating out the unobservable 
variable 

ly2d1= just takes the logaritmo neperiano of py2d1


*/

aaa=xp_y2d1*betaxp;
bbb=ones(obs_y2d1,r).*aaa;
ccc=bbb+u_y2d1;
pd1=cdfn(ccc);

ddd2=z2_y2d1*betaz2;
ddd3=z2_y2d1*betaz3;

fff2=ones(obs_y2d1,r).*ddd2;
fff3=ones(obs_y2d1,r).*ddd3;

ggg2=fff2+u_y2d1*beta_lo2;
ggg3=fff3+u_y2d1*beta_lo3;


py2=exp(ggg2)./(1+exp(ggg2)+exp(ggg3));

py2d1_n=pd1.*py2;
py2d1=meanc(py2d1_n');
ly2d1=ln(py2d1);

clear aaa, bbb, ccc, pd1, ddd2, ddd3, fff2, fff3, ggg2, ggg3, py2, py2d1_n;

@-------------------------------------------------   @
@Getting the log likelihood of event y2==1 and d==0  @
@-------------------------------------------------   @

aaa=xp_y2d0*betaxp;
bbb=ones(obs_y2d0,r).*aaa;
ccc=bbb+u_y2d0;
pd0=1-cdfn(ccc);

ddd2=z2_y2d0*betaz2;
ddd3=z2_y2d0*betaz3;

fff2=ones(obs_y2d0,r).*ddd2;
fff3=ones(obs_y2d0,r).*ddd3;

ggg2=fff2+u_y2d0*beta_lo2;
ggg3=fff3+u_y2d0*beta_lo3;


py2=exp(ggg2)./(1+exp(ggg2)+exp(ggg3));

py2d0_n=pd0.*py2;
py2d0=meanc(py2d0_n');
ly2d0=ln(py2d0);


clear aaa, bbb, ccc, pd0, ddd2, ddd2, fff2, fff3, ggg2, ggg3, py2, py2d0_n;

@-------------------------------------------------   @
@Getting the log likelihood of event y3==1 and d==0  @
@-------------------------------------------------   @

aaa=xp_y3d0*betaxp;
bbb=ones(obs_y3d0,r).*aaa;
ccc=bbb+u_y3d0;
pd0=1-cdfn(ccc);

ddd2=z2_y3d0*betaz2;
ddd3=z2_y3d0*betaz3;

fff2=ones(obs_y3d0,r).*ddd2;
fff3=ones(obs_y3d0,r).*ddd3;

ggg2=fff2+u_y3d0*beta_lo2;
ggg3=fff3+u_y3d0*beta_lo3;


py3=exp(ggg3)./(1+exp(ggg2)+exp(ggg3));

py3d0_n=pd0.*py3;
py3d0=meanc(py3d0_n');
ly3d0=ln(py3d0);

clear aaa, bbb, ccc, pd0, ddd2, ddd3, fff2, fff3, ggg2, ggg3, py3, py3d0_n;

@-------------------------------------------------   @
@Getting the log likelihood of event y3==1 and d==1  @
@-------------------------------------------------   @

aaa=xp_y3d1*betaxp;
bbb=ones(obs_y3d1,r).*aaa;
ccc=bbb+u_y3d1;
pd1=cdfn(ccc);

ddd2=z2_y3d1*betaz2;
ddd3=z2_y3d1*betaz3;

fff2=ones(obs_y3d1,r).*ddd2;
fff3=ones(obs_y3d1,r).*ddd3;

ggg2=fff2+u_y3d1*beta_lo2;
ggg3=fff3+u_y3d1*beta_lo3;


py3=exp(ggg3)./(1+exp(ggg2)+exp(ggg3));

py3d1_n=pd1.*py3;
py3d1=meanc(py3d1_n');
ly3d1=ln(py3d1);

clear aaa, bbb, ccc, pd1, ddd2, ddd3, fff2, fff3, ggg2, ggg3, py3, py3d1_n;


@-------------------------------------------------   @
@Getting the log likelihood of event y1==1 and d==1  @
@-------------------------------------------------   @

aaa=xp_y1d1*betaxp;
bbb=ones(obs_y1d1,r).*aaa;
ccc=bbb+u_y1d1;
pd1=cdfn(ccc);

ddd2=z2_y1d1*betaz2;
ddd3=z2_y1d1*betaz3;

fff2=ones(obs_y1d1,r).*ddd2;
fff3=ones(obs_y1d1,r).*ddd3;

ggg2=fff2+u_y1d1*beta_lo2;
ggg3=fff3+u_y1d1*beta_lo3;

@This is because y1 is the default category@
py1=1-exp(ggg2)./(1+exp(ggg2)+exp(ggg3))-exp(ggg3)./(1+exp(ggg2)+exp(ggg3));

py1d1_n=pd1.*py1;
py1d1=meanc(py1d1_n');
ly1d1=ln(py1d1);


clear aaa, bbb, ccc, pd1, ddd2, ddd3, fff2, fff3, ggg2, ggg3, py1, py1d1_n;

@-------------------------------------------------   @
@Getting the log likelihood of event y1==1 and d==0  @
@-------------------------------------------------   @

aaa=xp_y1d0*betaxp;
bbb=ones(obs_y1d0,r).*aaa;
ccc=bbb+u_y1d0;
pd0=1-cdfn(ccc);

ddd2=z2_y1d0*betaz2;
ddd3=z2_y1d0*betaz3;

fff2=ones(obs_y1d0,r).*ddd2;
fff3=ones(obs_y1d0,r).*ddd3;

ggg2=fff2+u_y1d0*beta_lo2;
ggg3=fff3+u_y1d0*beta_lo3;

@This is because y1 is the default category@
py1=1-exp(ggg2)./(1+exp(ggg2)+exp(ggg3))-exp(ggg3)./(1+exp(ggg2)+exp(ggg3));

py1d0_n=pd0.*py1;
py1d0=meanc(py1d0_n');
ly1d0=ln(py1d0);

clear aaa, bbb, ccc, pd0, ddd2, ddd3, fff2, fff3, ggg2, ggg3, py1, py1d0_n;

loglike=ly2d1|ly2d0|ly3d1|ly3d0|ly1d1|ly1d0;


 if add==1;

        retp(-sumc(loglike));
    elseif add==0;

       retp(loglike);

    endif;

endp;


